@@ -28,8 +28,9 @@ impl TreeSitterParser {
                 .include(&dir)
                 .cpp(true)
                 .flag("--std=c++14")
-                .flag("-Wno-unused-parameter")
-                .flag("-Wno-ignored-qualifiers");
+                .flag_if_supported("-Wno-implicit-fallthrough")
+                .flag_if_supported("-Wno-unused-parameter")
+                .flag_if_supported("-Wno-ignored-qualifiers");
             for file in cpp_files {
                 cpp_build.file(dir.join(file));
             }
@@ -46,11 +47,12 @@ impl TreeSitterParser {
 }
 
 fn main() {
-    // Only rerun if files in the vendor/ directory change.
-    // TODO: use specific source directories instead.
-    println!("cargo:rerun-if-changed=vendor");
-
     let parsers = vec![
+        TreeSitterParser {
+            name: "tree-sitter-bash".into(),
+            src_dir: "vendor/tree-sitter-bash-src".into(),
+            extra_files: vec!["scanner.cc".into()],
+        },
         TreeSitterParser {
             name: "tree-sitter-c".into(),
             src_dir: "vendor/tree-sitter-c-src".into(),
@@ -71,6 +73,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-clojure-src".into(),
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-commonlisp".into(),
+            src_dir: "vendor/tree-sitter-commonlisp-src".into(),
+            extra_files: vec![],
+        },
         TreeSitterParser {
             name: "tree-sitter-css".into(),
             src_dir: "vendor/tree-sitter-css-src".into(),
@@ -126,6 +133,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-python-src".into(),
             extra_files: vec!["scanner.cc".into()],
         },
+        TreeSitterParser {
+            name: "tree-sitter-ruby".into(),
+            src_dir: "vendor/tree-sitter-ruby-src".into(),
+            extra_files: vec!["scanner.cc".into()],
+        },
         TreeSitterParser {
             name: "tree-sitter-rust".into(),
             src_dir: "vendor/tree-sitter-rust-src".into(),
@@ -143,5 +155,11 @@ fn main() {
         },
     ];
 
+    // Only rerun if relevant files in the vendor/ directory change.
+    println!("cargo:rerun-if-changed=vendor/highlights");
+    for parser in &parsers {
+        println!("cargo:rerun-if-changed={}", parser.src_dir);
+    }
+
     parsers.par_iter().for_each(|p| p.build());
 }
