@@ -1,32 +1,53 @@
+//! Difftastic is a syntactic diff tool.
+//!
+//! For usage instructions and advice on contributing, see [the
+//! manual](http://difftastic.wilfred.me.uk/).
+//!
+
+mod context;
 mod dijkstra;
 mod files;
+mod graph;
+mod hunks;
 mod inline;
-mod intervals;
 mod line_parser;
 mod lines;
 mod positions;
-mod regex_parser;
 mod side_by_side;
 mod style;
 mod syntax;
 mod tree_sitter_parser;
-use atty::Stream;
-use clap::{crate_version, App, AppSettings, Arg};
-use std::env;
-use std::ffi::OsStr;
-use std::path::Path;
-use typed_arena::Arena;
 
-use crate::dijkstra::mark_syntax;
-use crate::files::{is_probably_binary, read_or_die};
-use crate::lines::{join_overlapping, visible_groups, MaxLine};
-use crate::syntax::{change_positions, init_info, init_info_single, matching_lines};
-use crate::tree_sitter_parser as tsp;
-
-extern crate pretty_env_logger;
 #[macro_use]
 extern crate log;
 
+use crate::hunks::{matched_pos_to_hunks, merge_adjacent};
+use log::info;
+use mimalloc::MiMalloc;
+
+/// The global allocator used by difftastic.
+///
+/// Diffing allocates a large amount of memory, and MiMalloc performs
+/// better.
+#[global_allocator]
+static GLOBAL: MiMalloc = MiMalloc;
+
+use atty::Stream;
+use clap::{crate_version, App, AppSettings, Arg};
+use std::{env, ffi::OsStr, path::Path};
+use typed_arena::Arena;
+use walkdir::WalkDir;
+
+use crate::{
+    dijkstra::mark_syntax,
+    files::{is_probably_binary, read_or_die},
+    lines::MaxLine,
+    syntax::{change_positions, init_info},
+    tree_sitter_parser as tsp,
+};
+
+extern crate pretty_env_logger;
+
 fn configure_color() {
     if atty::is(Stream::Stdout) || env::var("GIT_PAGER_IN_USE").is_ok() {
         // Always enable colour if stdout is a TTY or if the git pager is active.
@@ -51,6 +72,7 @@ enum Mode {
     },
 }
 
+/// Parse CLI arguments passed to the binary.
 fn parse_args() -> Mode {
     let matches =
         App::new("Difftastic")
@@ -68,6 +90,7 @@ fn parse_args() -> Mode {
             .get_matches();
 
     let args: Vec<_> = matches.values_of_lossy("positional_args").unwrap();
+    info!("CLI arguments: {:?}", args);
 
     if matches.is_present("dump-syntax") {
         if args.len() == 1 {
@@ -150,15 +173,14 @@ fn reset_sigpipe() {
     // Do nothing.
 }
 
+/// The entrypoint.
 fn main() {
     pretty_env_logger::init();
 
     reset_sigpipe();
     configure_color();
-    let arena = Arena::new();
 
-    let mode = parse_args();
-    let (display_path, lhs_path, rhs_path) = match mode {
+    match parse_args() {
         Mode::DumpTreeSitter { path } => {
             let extension = Path::new(&path).extension();
             let extension = extension.unwrap_or_else(|| OsStr::new(""));
@@ -166,14 +188,13 @@ fn main() {
                 Some(ts_lang) => {
                     let bytes = read_or_die(&path);
                     let src = String::from_utf8_lossy(&bytes).to_string();
-                    let tree = tsp::parse_to_tree(&src, &ts_lang);
-                    tsp::print_tree(tree);
+                    let (tree, _) = tsp::parse_to_tree(&src, &ts_lang);
+                    tsp::print_tree(&tree);
                 }
                 None => {
                     println!("No tree-sitter parser for extension: {:?}", extension);
                 }
             }
-            return;
         }
         Mode::DumpSyntax { path } => {
             let extension = Path::new(&path).extension();
@@ -182,30 +203,40 @@ fn main() {
                 Some(ts_lang) => {
                     let bytes = read_or_die(&path);
                     let src = String::from_utf8_lossy(&bytes).to_string();
+                    let arena = Arena::new();
                     let ast = tsp::parse(&arena, &src, &ts_lang);
-                    init_info_single(&ast, 0);
+                    init_info(&ast, &[]);
                     println!("{:#?}", ast);
                 }
                 None => {
                     println!("No tree-sitter parser for extension: {:?}", extension);
                 }
             }
-            return;
         }
         Mode::Diff {
             display_path,
             lhs_path,
             rhs_path,
-        } => (display_path, lhs_path, rhs_path),
+        } => {
+            if Path::new(&lhs_path).is_dir() && Path::new(&rhs_path).is_dir() {
+                diff_directories(&lhs_path, &rhs_path);
+            } else {
+                diff_file(&display_path, &lhs_path, &rhs_path);
+            }
+        }
     };
+}
 
-    let lhs_bytes = read_or_die(&lhs_path);
-    let rhs_bytes = read_or_die(&rhs_path);
+/// Print a diff between two files.
+// TODO: prefer PathBuf to &str for paths.
+fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
+    let lhs_bytes = read_or_die(lhs_path);
+    let rhs_bytes = read_or_die(rhs_path);
 
     let lhs_binary = is_probably_binary(&lhs_bytes);
     let rhs_binary = is_probably_binary(&rhs_bytes);
     if lhs_binary || rhs_binary {
-        println!("{}", style::header(&display_path, "binary"));
+        print!("{}", style::header(display_path, 1, 1, "binary"));
         return;
     }
 
@@ -219,76 +250,91 @@ fn main() {
 
     let extension = Path::new(&display_path).extension();
     let extension = extension.unwrap_or_else(|| OsStr::new(""));
-    // Try tree-sitter parser first unless DFT_RX (difftastic regex)
-    // environment variable is set.
-    let ts_lang = if env::var("DFT_RX").is_ok() {
-        None
-    } else {
-        tsp::from_extension(extension)
-    };
+    let ts_lang = tsp::from_extension(extension);
 
+    let arena = Arena::new();
     let (lang_name, lhs, rhs) = match ts_lang {
         Some(ts_lang) => (
-            ts_lang.name.into(),
+            ts_lang.name,
             tsp::parse(&arena, &lhs_src, &ts_lang),
             tsp::parse(&arena, &rhs_src, &ts_lang),
         ),
-        None => match regex_parser::from_extension(extension) {
-            Some(lang) => (
-                format!("{} (regex parser)", lang.name),
-                regex_parser::parse(&arena, &lhs_src, &lang),
-                regex_parser::parse(&arena, &rhs_src, &lang),
-            ),
-            None => (
-                "text".into(),
-                line_parser::parse(&arena, &lhs_src),
-                line_parser::parse(&arena, &rhs_src),
-            ),
-        },
+        None => (
+            "text",
+            line_parser::parse(&arena, &lhs_src),
+            line_parser::parse(&arena, &rhs_src),
+        ),
     };
 
-    println!("{}", style::header(&display_path, &lang_name));
-
     init_info(&lhs, &rhs);
     mark_syntax(lhs.get(0).copied(), rhs.get(0).copied());
 
     let lhs_positions = change_positions(&lhs_src, &rhs_src, &lhs);
     let rhs_positions = change_positions(&rhs_src, &lhs_src, &rhs);
 
-    let lhs_matched_lines = matching_lines(&lhs);
+    let hunks = matched_pos_to_hunks(&lhs_positions, &rhs_positions);
+    let hunks = merge_adjacent(
+        &hunks,
+        &lhs_positions,
+        &rhs_positions,
+        lhs_src.max_line(),
+        rhs_src.max_line(),
+    );
 
-    let mut groups = visible_groups(&lhs_positions, &rhs_positions);
-    if groups.is_empty() {
+    if hunks.is_empty() {
+        println!("{}", style::header(display_path, 1, 1, lang_name));
         if lang_name == "text" {
-            println!("No changes.");
+            println!("No changes.\n");
         } else {
-            println!("No syntactic changes.");
+            println!("No syntactic changes.\n");
         }
         return;
     }
 
-    for group in &mut groups {
-        group.pad(3, lhs_src.max_line(), rhs_src.max_line());
-    }
-    groups = join_overlapping(groups);
-
     if env::var("INLINE").is_ok() {
-        print!(
+        println!("{}", style::header(display_path, 1, 1, lang_name));
+
+        println!(
             "{}",
-            inline::display(&lhs_src, &rhs_src, &lhs_positions, &rhs_positions, &groups)
+            inline::display(&lhs_src, &rhs_src, &lhs_positions, &rhs_positions, &hunks)
         );
     } else {
-        print!(
+        println!(
             "{}",
-            side_by_side::display(
+            side_by_side::display_hunks(
+                &hunks,
+                display_path,
+                lang_name,
                 &lhs_src,
                 &rhs_src,
                 &lhs_positions,
                 &rhs_positions,
-                &lhs_matched_lines,
-                &groups
             )
         );
     }
-    println!();
+}
+
+/// Given two directories that contain the files, compare them
+/// pairwise.
+///
+/// When more than one file is modified, the hg extdiff extension passes directory
+/// paths with the all the modified files.  fn
+fn diff_directories(lhs_dir: &str, rhs_dir: &str) {
+    for entry in WalkDir::new(lhs_dir).into_iter().filter_map(Result::ok) {
+        let lhs_path = entry.path();
+        if lhs_path.is_dir() {
+            continue;
+        }
+
+        info!("LHS path is {:?} inside {:?}", lhs_path, lhs_dir);
+
+        let rel_path = lhs_path.strip_prefix(lhs_dir).unwrap();
+        let rhs_path = Path::new(rhs_dir).join(rel_path);
+
+        diff_file(
+            &rel_path.to_string_lossy(),
+            &lhs_path.to_string_lossy(),
+            &rhs_path.to_string_lossy(),
+        );
+    }
 }
