@@ -246,10 +246,6 @@ impl<'a> Syntax<'a> {
         self.info().num_ancestors.get()
     }
 
-    pub fn change(&'a self) -> Option<ChangeKind<'a>> {
-        self.info().change.get()
-    }
-
     pub fn first_line(&self) -> Option<LineNumber> {
         let position = match self {
             List { open_position, .. } => open_position,
@@ -536,7 +532,7 @@ pub struct MatchedPos {
 // "foo bar" -> vec!["foo", " ", "bar"]
 fn split_words(s: &str) -> Vec<String> {
     lazy_static! {
-        static ref RE: Regex = Regex::new(r"[a-zA-Z0-9]+|\n|[^a-zA-Z0-9\n]+").unwrap();
+        static ref RE: Regex = Regex::new(r"[a-zA-Z0-9]+|\n|[^a-zA-Z0-9\n]").unwrap();
     }
 
     RE.find_iter(s).map(|m| m.as_str().to_owned()).collect()
@@ -795,6 +791,12 @@ mod tests {
     use super::*;
     use pretty_assertions::assert_eq;
 
+    impl<'a> Syntax<'a> {
+        pub fn change(&'a self) -> Option<ChangeKind<'a>> {
+            self.info().change.get()
+        }
+    }
+
     #[test]
     fn test_comment_and_atom_differ() {
         let pos = vec![SingleLineSpan {
@@ -900,6 +902,13 @@ mod tests {
         assert_eq!(res, vec!["example", ".", "com"])
     }
 
+    #[test]
+    fn test_split_words_punctuations() {
+        let s = "example..";
+        let res = split_words(s);
+        assert_eq!(res, vec!["example", ".", "."])
+    }
+
     #[test]
     fn test_split_words_treats_newline_separately() {
         let s = "example.\ncom";
