@@ -1,4 +1,4 @@
-//! Parse code with tree-sitter language implementations.
+//! Load and configure parsers written with tree-sitter.
 
 use std::{borrow::Borrow, collections::HashSet, ffi::OsStr};
 
@@ -10,29 +10,44 @@ use crate::{
     syntax::{AtomKind, Syntax},
 };
 
+/// Configuration for a tree-sitter parser.
 pub struct TreeSitterConfig {
+    /// The language name shown to the user.
     pub name: &'static str,
+
+    /// The tree-sitter language parser.
     pub language: Language,
-    // Tree-sitter nodes that we treat as indivisible atoms. This is
-    // particularly useful for strings, as some grammars use several
-    // nodes for a single string literal. We don't want to say
-    // e.g. the closing string delimiter moved, as it's confusing and
-    // not well-balanced syntax.
-    //
-    // This is also useful for when tree-sitter nodes don't include
-    // all the children in the source. This is known limitation of
-    // tree-sitter, and occurs more often for complex string syntax.
-    // https://github.com/tree-sitter/tree-sitter/issues/1156
+
+    /// Tree-sitter nodes that we treat as indivisible atoms.
+    ///
+    /// This is particularly useful for strings, as some grammars use
+    /// several nodes for a single string literal. We don't want to
+    /// say e.g. the closing string delimiter moved, as it's confusing
+    /// and not well-balanced syntax.
+    ///
+    /// This is also useful for when tree-sitter nodes don't include
+    /// all the children in the source. This is known limitation of
+    /// tree-sitter, and occurs more often for complex string syntax.
+    /// <https://github.com/tree-sitter/tree-sitter/issues/1156>
     atom_nodes: HashSet<&'static str>,
+
+    /// We want to consider delimiter tokens as part of lists, not
+    /// standalone atoms. Tree-sitter includes delimiter tokens, so
+    /// mark which token pairs we consider to be delimiters.
     delimiter_tokens: Vec<(&'static str, &'static str)>,
+
+    /// Tree-sitter queries used for syntax highlighting this
+    /// language.
     highlight_queries: &'static str,
 }
 
 extern "C" {
+    fn tree_sitter_bash() -> Language;
     fn tree_sitter_c() -> Language;
     fn tree_sitter_c_sharp() -> Language;
     fn tree_sitter_clojure() -> Language;
     fn tree_sitter_cpp() -> Language;
+    fn tree_sitter_commonlisp() -> Language;
     fn tree_sitter_css() -> Language;
     fn tree_sitter_elisp() -> Language;
     fn tree_sitter_elixir() -> Language;
@@ -44,6 +59,7 @@ extern "C" {
     fn tree_sitter_ocaml() -> Language;
     fn tree_sitter_ocaml_interface() -> Language;
     fn tree_sitter_python() -> Language;
+    fn tree_sitter_ruby() -> Language;
     fn tree_sitter_rust() -> Language;
     fn tree_sitter_tsx() -> Language;
     fn tree_sitter_typescript() -> Language;
@@ -52,7 +68,18 @@ extern "C" {
 pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
     // TODO: find a nice way to extract name and extension information
     // from the package.json in these parsers.
+    // TODO: consider using
+    // https://github.com/github/linguist/blob/master/lib/linguist/languages.yml
+    // as a source of extensions.
+    // TODO: support files without extensions, such as .bashrc.
     match extension.to_string_lossy().borrow() {
+        "bash" | "sh" => Some(TreeSitterConfig {
+            name: "Bash",
+            language: unsafe { tree_sitter_bash() },
+            atom_nodes: (vec!["string", "raw_string"]).into_iter().collect(),
+            delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]")]),
+            highlight_queries: include_str!("../vendor/highlights/bash.scm"),
+        }),
         "c" => Some(TreeSitterConfig {
             name: "C",
             language: unsafe { tree_sitter_c() },
@@ -68,7 +95,7 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
         //
         // TODO: allow users to override the association between
         // extensions and parses.
-        "cc" | "cpp" | "h" | "hpp" => Some(TreeSitterConfig {
+        "cc" | "cpp" | "h" | "hh" | "hpp" | "cxx" => Some(TreeSitterConfig {
             name: "C++",
             language: unsafe { tree_sitter_cpp() },
             // The C++ grammar extends the C grammar, so the node
@@ -109,11 +136,11 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
         "css" => Some(TreeSitterConfig {
             name: "CSS",
             language: unsafe { tree_sitter_css() },
-            atom_nodes: (vec!["integer_value"]).into_iter().collect(),
+            atom_nodes: (vec!["integer_value", "float_value"]).into_iter().collect(),
             delimiter_tokens: (vec![("{", "}"), ("(", ")")]),
             highlight_queries: include_str!("../vendor/highlights/css.scm"),
         }),
-        "el" => Some(TreeSitterConfig {
+        "el" | ".emacs" => Some(TreeSitterConfig {
             name: "Emacs Lisp",
             language: unsafe { tree_sitter_elisp() },
             atom_nodes: (vec![]).into_iter().collect(),
@@ -159,7 +186,9 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
         "cjs" | "js" | "jsx" | "mjs" => Some(TreeSitterConfig {
             name: "JavaScript",
             language: unsafe { tree_sitter_javascript() },
-            atom_nodes: (vec!["string"]).into_iter().collect(),
+            atom_nodes: (vec!["string", "template_string", "regex"])
+                .into_iter()
+                .collect(),
             delimiter_tokens: (vec![
                 ("[", "]"),
                 ("(", ")"),
@@ -178,6 +207,13 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
             delimiter_tokens: (vec![("{", "}"), ("[", "]")]),
             highlight_queries: include_str!("../vendor/highlights/json.scm"),
         }),
+        "lisp" | "lsp" | "asd" => Some(TreeSitterConfig {
+            name: "Common Lisp",
+            language: unsafe { tree_sitter_commonlisp() },
+            atom_nodes: (vec!["str_lit"]).into_iter().collect(),
+            delimiter_tokens: (vec![("(", ")")]),
+            highlight_queries: "",
+        }),
         "ml" => Some(TreeSitterConfig {
             name: "OCaml",
             language: unsafe { tree_sitter_ocaml() },
@@ -193,13 +229,30 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
             delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
             highlight_queries: include_str!("../vendor/highlights/ocaml.scm"),
         }),
-        "py" => Some(TreeSitterConfig {
+        "py" | "py3" | "pyi" | "TARGETS" | "BUCK" | "bzl" => Some(TreeSitterConfig {
             name: "Python",
             language: unsafe { tree_sitter_python() },
             atom_nodes: (vec!["string"]).into_iter().collect(),
             delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
             highlight_queries: include_str!("../vendor/highlights/python.scm"),
         }),
+        "rb" | "spec" | "rake" => Some(TreeSitterConfig {
+            name: "Ruby",
+            language: unsafe { tree_sitter_ruby() },
+            atom_nodes: (vec!["string", "heredoc_body", "regex"])
+                .into_iter()
+                .collect(),
+            delimiter_tokens: (vec![
+                ("{", "}"),
+                ("(", ")"),
+                ("[", "]"),
+                ("|", "|"),
+                ("def", "end"),
+                ("begin", "end"),
+                ("class", "end"),
+            ]),
+            highlight_queries: include_str!("../vendor/highlights/ruby.scm"),
+        }),
         "rs" => Some(TreeSitterConfig {
             name: "Rust",
             language: unsafe { tree_sitter_rust() },
@@ -212,7 +265,9 @@ pub fn from_extension(extension: &OsStr) -> Option<TreeSitterConfig> {
         "ts" => Some(TreeSitterConfig {
             name: "TypeScript",
             language: unsafe { tree_sitter_typescript() },
-            atom_nodes: (vec!["string", "template_string"]).into_iter().collect(),
+            atom_nodes: (vec!["string", "template_string", "regex"])
+                .into_iter()
+                .collect(),
             delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
             highlight_queries: include_str!("../vendor/highlights/typescript.scm"),
         }),
@@ -396,6 +451,17 @@ fn syntax_from_cursor<'a>(
 ) -> &'a Syntax<'a> {
     let node = cursor.node();
 
+    if node.is_error() {
+        let position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
+        let content = &src[node.start_byte()..node.end_byte()];
+
+        warn!(
+            "Tree-sitter syntax error at {:?}: {}",
+            position.get(0),
+            content
+        );
+    }
+
     if config.atom_nodes.contains(node.kind()) {
         // Treat nodes like string literals as atoms, regardless
         // of whether they have children.
